package javax.swing;

import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.Element;
import javax.swing.text.Style;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.StyleConstants;
import java.awt.Component;
import javax.swing.text.AttributeSet;
import javax.swing.text.Caret;
import javax.swing.text.BadLocationException;
import javax.swing.text.AbstractDocument;
import javax.swing.text.Document;
import javax.swing.text.StyledDocument;
import javax.swing.text.EditorKit;

public class JTextPane extends JEditorPane
{
    private static final String uiClassID = "TextPaneUI";
    
    public JTextPane() {
        EditorKit defaultEditorKit = this.createDefaultEditorKit();
        String contentType = defaultEditorKit.getContentType();
        if (contentType != null && JEditorPane.getEditorKitClassNameForContentType(contentType) == JTextPane.defaultEditorKitMap.get(contentType)) {
            this.setEditorKitForContentType(contentType, defaultEditorKit);
        }
        this.setEditorKit(defaultEditorKit);
    }
    
    public JTextPane(StyledDocument styledDocument) {
        this();
        this.setStyledDocument(styledDocument);
    }
    
    @Override
    public String getUIClassID() {
        return "TextPaneUI";
    }
    
    @Override
    public void setDocument(Document document) {
        if ((document instanceof StyledDocument)) {
            super.setDocument(document);
            return;
        }
        throw new IllegalArgumentException("Model must be StyledDocument");
    }
    
    public void setStyledDocument(StyledDocument document) {
        super.setDocument(document);
    }
    
    public StyledDocument getStyledDocument() {
        return (StyledDocument)this.getDocument();
    }
    
    @Override
    public void replaceSelection(String s) {
        this.replaceSelection(s, true);
    }
    
    private void replaceSelection(String text, boolean b) {
        if (b && !this.isEditable()) {
            UIManager.getLookAndFeel().provideErrorFeedback(this);
            return;
        }
        StyledDocument styledDocument = this.getStyledDocument();
        if (styledDocument != null) {
            try {
                Caret caret = this.getCaret();
                int saveComposedText = this.saveComposedText(caret.getDot());
                int min = Math.min(caret.getDot(), caret.getMark());
                int max = Math.max(caret.getDot(), caret.getMark());
                AttributeSet copyAttributes = this.getInputAttributes().copyAttributes();
                if ((styledDocument instanceof AbstractDocument)) {
                    ((AbstractDocument)styledDocument).replace(min, max - min, text, copyAttributes);
                }
                else {
                    if (min != max) {
                        styledDocument.remove(min, max - min);
                    }
                    if (text != null && text.length() > 0) {
                        styledDocument.insertString(min, text, copyAttributes);
                    }
                }
                if (!(saveComposedText == 0)) {
                    this.restoreComposedText();
                }
            }
            catch (BadLocationException ex) {
                UIManager.getLookAndFeel().provideErrorFeedback(this);
            }
        }
        return;
    }
    
    public void insertComponent(Component c) {
        MutableAttributeSet inputAttributes = this.getInputAttributes();
        inputAttributes.removeAttributes(inputAttributes);
        StyleConstants.setComponent(inputAttributes, c);
        this.replaceSelection(" ", false);
        inputAttributes.removeAttributes(inputAttributes);
    }
    
    public void insertIcon(Icon c) {
        MutableAttributeSet inputAttributes = this.getInputAttributes();
        inputAttributes.removeAttributes(inputAttributes);
        StyleConstants.setIcon(inputAttributes, c);
        this.replaceSelection(" ", false);
        inputAttributes.removeAttributes(inputAttributes);
    }
    
    public Style addStyle(String s, Style style) {
        return this.getStyledDocument().addStyle(s, style);
    }
    
    public void removeStyle(String s) {
        this.getStyledDocument().removeStyle(s);
    }
    
    public Style getStyle(String s) {
        return this.getStyledDocument().getStyle(s);
    }
    
    public void setLogicalStyle(Style style) {
        this.getStyledDocument().setLogicalStyle(this.getCaretPosition(), style);
    }
    
    public Style getLogicalStyle() {
        return this.getStyledDocument().getLogicalStyle(this.getCaretPosition());
    }
    
    public AttributeSet getCharacterAttributes() {
        Element characterElement = this.getStyledDocument().getCharacterElement(this.getCaretPosition());
        if (characterElement != null) {
            return characterElement.getAttributes();
        }
        return null;
    }
    
    public void setCharacterAttributes(AttributeSet set, boolean b) {
        int selectionStart = this.getSelectionStart();
        int selectionEnd = this.getSelectionEnd();
        if (selectionStart != selectionEnd) {
            this.getStyledDocument().setCharacterAttributes(selectionStart, selectionEnd - selectionStart, set, b);
        }
        else {
            MutableAttributeSet inputAttributes = this.getInputAttributes();
            if (b) {
                inputAttributes.removeAttributes(inputAttributes);
            }
            inputAttributes.addAttributes(set);
        }
    }
    
    public AttributeSet getParagraphAttributes() {
        Element paragraphElement = this.getStyledDocument().getParagraphElement(this.getCaretPosition());
        if (paragraphElement != null) {
            return paragraphElement.getAttributes();
        }
        return null;
    }
    
    public void setParagraphAttributes(AttributeSet set, boolean b) {
        int selectionStart = this.getSelectionStart();
        this.getStyledDocument().setParagraphAttributes(selectionStart, this.getSelectionEnd() - selectionStart, set, b);
    }
    
    public MutableAttributeSet getInputAttributes() {
        return this.getStyledEditorKit().getInputAttributes();
    }
    
    protected final StyledEditorKit getStyledEditorKit() {
        return (StyledEditorKit)this.getEditorKit();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("TextPaneUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected EditorKit createDefaultEditorKit() {
        return new StyledEditorKit();
    }
    
    @Override
    public final void setEditorKit(EditorKit editorKit) {
        if ((editorKit instanceof StyledEditorKit)) {
            super.setEditorKit(editorKit);
            return;
        }
        throw new IllegalArgumentException("Must be StyledEditorKit");
    }
    
    @Override
    protected String paramString() {
        return super.paramString();
    }
}
